<?php

/**
 * ECollapse makes a HTML DOM element collapsible as specified by passed options
 *
 * Based on the jQuery-Collapse script
 * @link http://webcloud.se/code/jQuery-Collapse/
 *
 * @author Luke Jurgs
 * @version 0.0.2-2012-03-03
 */
class ECollapse extends CWidget {

	private $_assetUrl;

	/**
	 * The selector to apply the behaviour to, defaults to '.collaspe'.
	 *
	 * @var string $selector
	 */
	public $selector = '.collapse';

	/**
	 * Should the selected element start in a collapsed state, defaults to false.
	 *
	 * @var boolean
	 */
	public $collapsed = true;

	/**
	 * The duration of the default animation, replaces the string '{duration}' in $show and $hide,
	 * valid values are: 'fast', 'slow' or a integer specifying milliseconds, defaults to 'fast'.
	 *
	 * @var mixed $speed
	 */
	public $duration = 'fast';

	/**
	 * The CSS file used for the widget. Defaults to the included CSS file.
	 *
	 * @see assets/css/jquery.collapse.css
	 *
	 * @var string $cssFile
	 */
	public $cssFile;

	/**
	 * A function that defines how the content should be visually expanded.
	 *
	 * @var string $show
	 */
	public $show = 'js:
		function() {
			this.animate({
				opacity: "toggle",
				height: "toggle"
			}, "{duration}");
		}';

	/**
	 * A function that defines how the content should be visually hidden.
	 *
	 * @var string $hide
	 */
	public $hide = 'js:
			function() {
				this.animate({
					opacity: "toggle",
					height: "toggle"
				}, "{duration}");
			}';

	/**
	 * Type of element you want act as clickable headings, defaults to 'h3'.
	 *
	 * @var string $head
	 */
	public $head = 'h3';

	/**
	 * Type of element to group hidden content, defaults to 'div, ul'.
	 *
	 * @var string $group
	 */
	public $group = 'div, ul';

	/**
	 * Name of cookie used by the plugin, defaults to 'collapse'. Setting this to false will disable cookies
	 * for this widget.
	 *
	 * @var mixed $cookieName
	 */
	public $cookieName = 'collapse';

	/**
	 * Initializes the widget.
	 * This method is called by {@link CBaseController::createWidget}
	 * and {@link CBaseController::beginWidget} after the widget's
	 * properties have been initialized.
	 */
	public function init() {
		$this->_assetUrl = Yii::app()->assetManager->publish(dirname(__FILE__) . DIRECTORY_SEPARATOR . 'assets');
		//setup the show and hide functions if they are set
		$this->show = str_replace('{duration}', $this->duration, $this->show);
		$this->hide = str_replace('{duration}', $this->duration, $this->hide);
	}

	/**
	 * Executes the widget.
	 * This method is called by {@link CBaseController::endWidget}.
	 */
	public function run() {
		$this->registerClientScript();
	}

	/**
	 * Registers necessary client scripts.
	 */
	public function registerClientScript() {
		$clientScript = Yii::app()->clientScript;
		//register javascript
		$clientScript->registerCoreScript('jquery');
		$clientScript->registerScriptFile($this->_assetUrl . '/js/jquery.cookie.js');
		$clientScript->registerScriptFile($this->_assetUrl . '/js/jquery.collapse.js');
		$javascript = '';
		//register css
		if (null === $this->cssFile) {
			$clientScript->registerCssFile($this->_assetUrl . '/css/jquery.collapse.css');
			//apply the class in the default css to the head elements
			$javascript .= 'jQuery("' . $this->selector . ' ' . $this->head . '").addClass("jquery-collapse-head");';
		} else {
			$clientScript->registerCssFile($this->cssFile);
		}

		$id = __CLASS__ . '#' . $this->getId();

		//add a class to the html element that lets us hide the groups so they don't flash
		$clientScript->registerScript('jquery-collapse-hide-group-script', 'document.documentElement.className = "js";', CClientScript::POS_HEAD);
		$groups = explode(',', $this->group);
		foreach ($groups as &$group) {
			$group = '.js ' . $this->selector . ' .inactive ~ ' . trim($group) . '.hide';
		}
		$groups = implode(', ', $groups);
		$clientScript->registerCss($id . 'jquery-collapse-hide-group-css', $groups . ' { display : none; }');

		//build the jquery collapsed script
		$options = array(
			'head' => $this->head,
			'group' => $this->group,
			'cookieName' => $this->cookieName,
		);

		if (!empty($this->show)) {
			$options['show'] = $this->show;
		}
		if (!empty($this->hide)) {
			$options['hide'] = $this->hide;
		}

		$options = CJavaScript::encode($options);

		//set the class of head elements to inactive
		if ($this->collapsed) {
			$javascript .= '
				jQuery("' . $this->selector . ' ' . $this->head . '").each(function() {
					if (!$(this).hasClass("active") && !$(this).hasClass("inactive")) {
						$(this).addClass("inactive");
					}
				});
			';
		} else {
			$javascript .= '
				jQuery("' . $this->selector . ' ' . $this->head . '").each(function() {
					if (!$(this).hasClass("active") && !$(this).hasClass("inactive")) {
						$(this).addClass("active");
					}
				});
			';
		}
		$javascript .= 'jQuery("' . $this->selector . '").collapse(' . $options . ');';
		$clientScript->registerScript($id, $javascript);
	}

}